@@ -11,12 +11,13 @@ use SPVM::Data;
 use SPVM::Data::Array;
 
 use SPVM::Build;
+use SPVM::PerlAPI;
 
-use Encode 'encode';
+use Encode 'encode', 'decode';
 
 use Carp 'confess';
 
-our $VERSION = '0.0366';
+our $VERSION = '0.0367';
 
 our $ENV;
 our $BUILD;
@@ -55,174 +56,44 @@ CHECK {
     unless ($compile_success) {
       die "SPVM compile error";
     }
+    
+    # Set env
+    $SPVM::ENV = $BUILD->{env};
   }
 }
 
-sub new_byte_array {
-  my $elements = shift;
-  
-  return undef unless defined $elements;
-  
-  if (ref $elements ne 'ARRAY') {
-    confess "Argument must be array reference";
-  }
-  
-  my $length = @$elements;
-  
-  my $array = SPVM::new_byte_array_len($length);
-  
-  $array->set_elements($elements);
-  
-  return $array;
-}
+sub new_object_array { SPVM::PerlAPI::new_object_array($SPVM::ENV, @_) }
+sub new_value_t_array { SPVM::PerlAPI::new_value_t_array($SPVM::ENV, @_) }
+sub new_object { SPVM::PerlAPI::new_object($SPVM::ENV, @_) }
 
-sub new_byte_array_string {
-  my $string = shift;
-  
-  # Encode internal string to UTF-8 string
-  my $bin = Encode::encode('UTF-8', $string);
-  
-  my $length = length $bin;
-  
-  my $array = SPVM::new_byte_array_len($length);
-  $array->set_bin($bin);
-  
-  return $array;
-}
+sub new_byte_array_len { SPVM::PerlAPI::new_byte_array_len($SPVM::ENV, @_) }
+sub new_short_array_len { SPVM::PerlAPI::new_short_array_len($SPVM::ENV, @_) }
+sub new_int_array_len { SPVM::PerlAPI::new_int_array_len($SPVM::ENV, @_) }
+sub new_long_array_len { SPVM::PerlAPI::new_long_array_len($SPVM::ENV, @_) }
+sub new_float_array_len { SPVM::PerlAPI::new_float_array_len($SPVM::ENV, @_) }
+sub new_double_array_len { SPVM::PerlAPI::new_double_array_len($SPVM::ENV, @_) }
+sub new_object_array_len { SPVM::PerlAPI::new_object_array_len($SPVM::ENV, @_) }
+sub new_multi_array_len { SPVM::PerlAPI::new_multi_array_len($SPVM::ENV, @_) }
+sub new_value_t_array_len { SPVM::PerlAPI::new_value_t_array_len($SPVM::ENV, @_) }
+sub set_exception_undef { SPVM::PerlAPI::set_exception_undef($SPVM::ENV, @_) }
+sub get_memory_blocks_count { SPVM::PerlAPI::get_memory_blocks_count($SPVM::ENV, @_) }
+sub call_sub { SPVM::PerlAPI::call_sub($SPVM::ENV, @_) }
 
-sub new_short_array {
-  my $elements = shift;
+sub new_byte_array { SPVM::PerlAPI::new_byte_array($SPVM::ENV, @_) }
+sub new_byte_array_string { SPVM::PerlAPI::new_byte_array_string($SPVM::ENV, @_) }
+sub new_short_array { SPVM::PerlAPI::new_short_array($SPVM::ENV, @_) }
+sub new_int_array { SPVM::PerlAPI::new_int_array($SPVM::ENV, @_) }
+sub new_long_array { SPVM::PerlAPI::new_long_array($SPVM::ENV, @_) }
+sub new_float_array { SPVM::PerlAPI::new_float_array($SPVM::ENV, @_) }
+sub new_double_array { SPVM::PerlAPI::new_double_array($SPVM::ENV, @_) }
 
-  return undef unless defined $elements;
-
-  if (ref $elements ne 'ARRAY') {
-    confess "Argument must be array reference";
-  }
-  
-  my $length = @$elements;
-  
-  my $array = SPVM::new_short_array_len($length);
-  
-  $array->set_elements($elements);
-  
-  return $array;
-}
-
-sub new_int_array {
-  my $elements = shift;
-
-  return undef unless defined $elements;
-
-  if (ref $elements ne 'ARRAY') {
-    confess "Argument must be array reference";
-  }
-  
-  my $length = @$elements;
-  
-  my $array = SPVM::new_int_array_len($length);
-  
-  $array->set_elements($elements);
-  
-  return $array;
-}
-
-sub new_long_array {
-  my $elements = shift;
-
-  return undef unless defined $elements;
-
-  if (ref $elements ne 'ARRAY') {
-    confess "Argument must be array reference";
-  }
-  
-  my $length = @$elements;
-  
-  my $array = SPVM::new_long_array_len($length);
-  
-  $array->set_elements($elements);
-  
-  return $array;
-}
-
-sub new_float_array {
-  my $elements = shift;
-
-  return undef unless defined $elements;
-
-  if (ref $elements ne 'ARRAY') {
-    confess "Argument must be array reference";
-  }
-  
-  my $length = @$elements;
-  
-  my $array = SPVM::new_float_array_len($length);
-  
-  $array->set_elements($elements);
-  
-  return $array;
-}
-
-sub new_double_array {
-  my $elements = shift;
-  
-  return undef unless defined $elements;
-  
-  if (ref $elements ne 'ARRAY') {
-    confess "Argument must be array reference";
-  }
-  
-  my $length = @$elements;
-  
-  my $array = SPVM::new_double_array_len($length);
-  
-  $array->set_elements($elements);
-  
-  return $array;
-}
-
-sub new_object_array {
-  my ($package_name, $elements) = @_;
-  
-  return undef unless defined $elements;
-  
-  if (ref $elements ne 'ARRAY') {
-    confess "Argument must be array reference";
-  }
-  
-  my $length = @$elements;
-  
-  my $array = SPVM::new_object_array_len($package_name, $length);
-  
-  $array->set_elements($elements);
-  
-  return $array;
-}
-
-sub new_value_t_array {
-  my ($package_name, $elements) = @_;
-  
-  return undef unless defined $elements;
-  
-  if (ref $elements ne 'ARRAY') {
-    confess "Argument must be array reference";
-  }
-  
-  my $length = @$elements;
-  
-  my $array = SPVM::new_value_t_array_len($package_name, $length);
-
-  $array->set_elements($elements);
-  
-  return $array;
-}
-
-sub new_object {
-  my $package_name = shift;
-  
-  my $object = SPVM::Data::Package->new($package_name);
-  
-  return $object;
-}
+sub set_array_elements { SPVM::PerlAPI::set_array_elements($SPVM::ENV, @_) }
+sub set_array_elements_bin { SPVM::PerlAPI::set_array_elements_bin($SPVM::ENV, @_) }
+sub set_array_element { SPVM::PerlAPI::set_array_element($SPVM::ENV, @_) }
+sub get_array_element { SPVM::PerlAPI::get_array_element($SPVM::ENV, @_) }
+sub get_array_elements { SPVM::PerlAPI::get_array_elements($SPVM::ENV, @_) }
+sub get_array_elements_bin { SPVM::PerlAPI::get_array_elements_bin($SPVM::ENV, @_) }
+sub get_array_elements_string { SPVM::PerlAPI::get_array_elements_string($SPVM::ENV, @_) }
 
 1;
 
